@@ -1,11 +1,12 @@
 use anyhow::{bail, Context, Result};
+use chrono::Duration;
 use chrono::Local;
 use chrono::NaiveDate;
+use dialoguer::MultiSelect;
 use genee::configuration;
 use genee::datafile;
 use genee::datafile::DiaryData;
 use genee::graphing;
-use std::io;
 use std::path::{Path, PathBuf};
 use structopt::StructOpt;
 
@@ -22,18 +23,6 @@ struct CliOptions {
     #[structopt(short, long)]
     graph_days: Option<usize>,
 
-    /// If set, habit information for all the missing days is queried between --append-date
-    /// and yesterday. If --append-date is not set, all the missing days are queried between the
-    /// first entry in the diary and yesterday.
-    #[structopt(short, long)]
-    fill: bool,
-
-    /// When provided, the habit data is queried and written to the diary at the specified date.
-    /// The format of the date must be YYYY-MM-DD.
-    /// If --fill is also set, this option serves a different purpose.
-    #[structopt(short, long)]
-    append_date: Option<String>,
-
     /// Specifies the number of displayed periods when graphing the diary data.
     /// When not provided, its value is loaded from persistent configuration file.
     #[structopt(short, long)]
@@ -48,73 +37,105 @@ struct CliOptions {
     #[structopt(long)]
     max_displayed_cols: Option<usize>,
 
-    /// If set, the current persistent configuration is displayed to the terminal.
-    #[structopt(long)]
-    list_config: bool,
+    #[structopt(subcommand)]
+    command: Command,
+}
 
-    /// If set, the provided values for --datafile --graph-days --past-periods --max-displayed-cols
-    /// and --list-previous-days options are written to the persistent configuration.
-    /// Unspecified options are reset to their default value.
-    #[structopt(long)]
-    save_config: bool,
+#[derive(StructOpt)]
+enum Command {
+    /// If set, habit information for all the missing days is queried between --from-date
+    /// and yesterday. If --from-date is not set, all the missing days are queried between the
+    /// first entry in the diary and yesterday.
+    /// If there is no entry in the diary, only yesterday is queried.
+    Fill {
+        /// Start the querying from this date. Must be in format YYYY-MM-DD.
+        #[structopt(long)]
+        from_date: Option<String>,
+
+        /// Controls whether to graph the diary after the fill.
+        #[structopt(long)]
+        no_graph: bool,
+    },
+
+    /// Displays the habit data according to the specified options to the terminal.
+    Graph,
+
+    /// Queries for habit information on the specified date.
+    Insert {
+        /// Date to set habit data on.  Must be in format YYYY-MM-DD.
+        date: String,
+
+        /// Controls whether to graph the diary after the fill.
+        #[structopt(long)]
+        no_graph: bool,
+    },
+
+    /// Prints the persistent configuration.
+    ListConfig,
 
     /// Provide a comma separated list of habit categories. A new diary file is created at the specified
-    /// --datafile path. Be aware that this overwrites any existing diary file.
-    #[structopt(long)]
-    new: Option<String>,
+    /// --datafile path.
+    New { category_list: String },
+
+    /// Saves the specified options to persistent configuration.
+    SaveConfig,
 }
 
 fn main() -> Result<()> {
-    let opt = CliOptions::from_args();
-    if opt.save_config {
-        save_config(&opt)?;
-    }
-    let persistent_config = configuration::load_config()?;
-    let opt = merge_cli_and_persistent_options(opt, &persistent_config);
-    if opt.list_config {
-        pretty_print_config()?;
-        return Ok(());
-    }
-    let datafile_path = opt.datafile.unwrap();
-    if opt.new.is_some() {
-        create_new(&datafile_path, opt.new.as_ref().unwrap())?;
-    }
-    let mut data = datafile::parse_csv_to_diary_data(&datafile_path)?;
-    let append_date = get_append_date(&opt.append_date)?;
-    let graph_date: NaiveDate;
-    if opt.fill {
-        graph_date = yesterday();
-        let appended_dates = datafile::get_missing_dates(&data, &append_date, &graph_date)?;
-        for date in appended_dates {
-            update_data(&mut data, &date)?;
+    let opt = handle_config()?;
+    let datafile_path = opt.datafile.as_ref().unwrap();
+    match opt.command {
+        Command::Fill {
+            ref from_date,
+            no_graph,
+        } => {
+            let from_date = parse_from_date(from_date)?;
+            let data = datafile::parse_csv_to_diary_data(datafile_path)?;
+            let to_date = get_graph_date(&data)?;
+            let data = fill_datafile(datafile_path, &from_date, &to_date, data)?;
+            if !no_graph {
+                plot_datafile(&opt, &to_date, &data)?;
+            }
+        }
+        Command::Graph => {
+            let data = datafile::parse_csv_to_diary_data(datafile_path)?;
+            plot_datafile(&opt, &Local::today().naive_local(), &data)?;
+        }
+        Command::Insert { ref date, no_graph } => {
+            let date = parse_from_date(&Some(date.clone()))?.unwrap();
+            let data = datafile::parse_csv_to_diary_data(datafile_path)?;
+            let data = insert_to_datafile(datafile_path, &date, data)?;
+            if !no_graph {
+                plot_datafile(&opt, &date, &data)?;
+            }
+        }
+        Command::ListConfig => {
+            let persistent_config = configuration::load_config()?;
+            println!(
+                "Listing persistent configuration loaded from '{}'\n{}",
+                configuration::get_config_path().to_string_lossy(),
+                &persistent_config
+            );
+        }
+        Command::New { ref category_list } => {
+            create_new(datafile_path, category_list)?;
+        }
+        Command::SaveConfig => {
+            save_config(&opt)?;
         }
-        datafile::serialize_to_csv(&datafile_path, &data)?;
-    } else if append_date.is_some() {
-        graph_date = append_date.unwrap();
-        update_data(&mut data, &graph_date)?;
-        datafile::serialize_to_csv(&datafile_path, &data)?;
-    } else {
-        graph_date = Local::today().naive_local();
     }
-    if opt.list_previous_days.unwrap() > 0 {
-        let today = Local::today().naive_local();
-        let start_day = today - chrono::Duration::days(opt.list_previous_days.unwrap() as i64);
-        print!(
-            "{}",
-            graphing::pretty_print_diary_rows(&data, &start_day, &today)
-        );
-    }
-    graphing::graph_last_n_days(
-        &data,
-        &graph_date,
-        opt.graph_days.unwrap(),
-        opt.past_periods.unwrap(),
-        opt.max_displayed_cols.unwrap(),
-    )?;
+
     Ok(())
 }
 
-fn get_append_date(input_date: &Option<String>) -> Result<Option<NaiveDate>> {
+fn handle_config() -> Result<CliOptions> {
+    let opt = CliOptions::from_args();
+    let persistent_config = configuration::load_config()?;
+    let opt = merge_cli_and_persistent_options(opt, &persistent_config);
+    Ok(opt)
+}
+
+fn parse_from_date(input_date: &Option<String>) -> Result<Option<NaiveDate>> {
     match input_date {
         Some(date_string) => Ok(Some(
             NaiveDate::parse_from_str(date_string, datafile::DATE_FORMAT).context(format!(
@@ -127,6 +148,18 @@ fn get_append_date(input_date: &Option<String>) -> Result<Option<NaiveDate>> {
     }
 }
 
+fn get_graph_date(data: &datafile::DiaryData) -> Result<NaiveDate> {
+    let today = Local::today().naive_local();
+    if data.data.contains_key(&today) {
+        Ok(today)
+    } else {
+        Local::today()
+            .naive_local()
+            .checked_sub_signed(chrono::Duration::days(1))
+            .ok_or_else(|| anyhow::Error::msg("Could not get yesterday"))
+    }
+}
+
 fn merge_cli_and_persistent_options(
     options_from_cli: CliOptions,
     persistent_config: &configuration::Config,
@@ -151,16 +184,6 @@ fn merge_cli_and_persistent_options(
     }
 }
 
-fn pretty_print_config() -> Result<()> {
-    let persistent_config = configuration::load_config()?;
-    println!(
-        "Listing persistent configuration loaded from \"{}\"\n{}",
-        configuration::get_config_path().to_string_lossy(),
-        configuration::pretty_print_config(&persistent_config)?
-    );
-    Ok(())
-}
-
 fn save_config(opt: &CliOptions) -> Result<()> {
     let provided_datafile_path = opt
         .datafile
@@ -188,47 +211,74 @@ fn save_config(opt: &CliOptions) -> Result<()> {
     Ok(())
 }
 
-fn update_data(data: &mut DiaryData, date: &NaiveDate) -> Result<()> {
-    let append_bools = input_data_interactively(date, &data.header);
-    match datafile::update_data(data, date, &append_bools)? {
-        datafile::SuccessfulUpdate::AddedNew => {
-            println!(
-                "Adding new row to datafile:\n{}",
-                graphing::pretty_print_diary_row(data, date)
-            );
-            Ok(())
-        }
-        datafile::SuccessfulUpdate::ReplacedExisting(_existing_row) => {
-            println!(
-                "Updated row in datafile:\n{}",
-                graphing::pretty_print_diary_row(data, date)
-            );
-            Ok(())
-        }
-    }
-}
-
-fn input_data_interactively(date: &NaiveDate, headers: &[String]) -> Vec<bool> {
-    println!(
+fn input_day_interactively(data: &mut DiaryData, date: &NaiveDate) -> Result<()> {
+    let prompt = format!(
         "Enter habit data for date {}",
         date.format(datafile::DATE_FORMAT)
     );
-    headers
-        .iter()
-        .map(|header| {
-            println!("{} ?", header);
-            let mut line = String::new();
-            let _count = io::stdin().read_line(&mut line);
-            !line.trim().is_empty()
-        })
-        .collect()
+    let selected_items = MultiSelect::new()
+        .with_prompt(prompt)
+        .items(&data.header)
+        .interact()?;
+
+    let mut append_bools = vec![false; data.header.len()];
+    for idx in selected_items {
+        append_bools[idx] = true;
+    }
+
+    datafile::update_data(data, date, &append_bools)?;
+    Ok(())
 }
 
-fn yesterday() -> NaiveDate {
-    Local::today()
-        .naive_local()
-        .checked_sub_signed(chrono::Duration::days(1))
-        .unwrap()
+fn fill_datafile(
+    datafile_path: &Path,
+    from_date: &Option<NaiveDate>,
+    to_date: &NaiveDate,
+    mut data: DiaryData,
+) -> Result<DiaryData> {
+    let mut missing_dates = datafile::get_missing_dates(&data, from_date, to_date);
+    if data.data.is_empty() {
+        missing_dates.push(
+            Local::today()
+                .naive_local()
+                .checked_sub_signed(Duration::days(1))
+                .unwrap(),
+        );
+    }
+    for date in missing_dates {
+        input_day_interactively(&mut data, &date)?;
+    }
+    datafile::serialize_to_csv(datafile_path, &data)?;
+    Ok(data)
+}
+
+fn insert_to_datafile(
+    datafile_path: &Path,
+    date: &NaiveDate,
+    mut data: DiaryData,
+) -> Result<DiaryData> {
+    input_day_interactively(&mut data, date)?;
+    datafile::serialize_to_csv(datafile_path, &data)?;
+    Ok(data)
+}
+
+fn plot_datafile(opt: &CliOptions, last_date: &NaiveDate, data: &DiaryData) -> Result<()> {
+    if opt.list_previous_days.unwrap() > 0 {
+        let start_day =
+            *last_date - chrono::Duration::days(opt.list_previous_days.unwrap() as i64 - 1i64);
+        print!(
+            "{}",
+            graphing::pretty_print_diary_rows(data, &start_day, last_date)
+        );
+    }
+    graphing::graph_last_n_days(
+        data,
+        last_date,
+        opt.graph_days.unwrap(),
+        opt.past_periods.unwrap(),
+        opt.max_displayed_cols.unwrap(),
+    )?;
+    Ok(())
 }
 
 fn create_new(path: &Path, headers_string: &str) -> Result<()> {
@@ -240,5 +290,6 @@ fn create_new(path: &Path, headers_string: &str) -> Result<()> {
         headers_vector.push(String::from(title));
     }
     datafile::create_new_csv(path, &headers_vector)?;
+    println!("New datafile successfully created at {}", path.display());
     Ok(())
 }
